import 'dart:convert';
import 'dart:io';

import 'package:chapa/core/chapa_error.dart';
import 'package:chapa/core/metrics/metric_manager.dart';
import 'package:chapa/models/requests/charge_card/validate_charge_request.dart';
import 'package:chapa/models/requests/verify_charge_request.dart';
import 'package:chapa/models/responses/charge_response.dart';
import 'package:chapa/utils/chapa_urls.dart';
import 'package:http/http.dart' as http;

/// Chapa Utility class
class ChapaAPIUtils {



  /// Validates payments with OTP
  /// returns an instance of ChargeResponse or throws an error
  static Future<ChargeResponse> validatePayment(
      String otp,
      String chpRef,
      http.Client client,
      final bool isDebugMode,
      final String publicKey,
      final isBankAccount,
      [String feature = ""]) async {
    final stopWatch = Stopwatch();

    final url = ChapaURLS.getBaseUrl(isDebugMode) +
        ChapaURLS.VALIDATE_CHARGE;
    final uri = Uri.parse(url);
    final ValidateChargeRequest chargeRequest =
        ValidateChargeRequest(otp, chpRef, isBankAccount);
    final payload = chargeRequest.toJson();
    try {
      final http.Response response = await client.post(uri,
          headers: {HttpHeaders.authorizationHeader: publicKey}, body: payload);

      if (feature.isNotEmpty) {
        MetricManager.logMetric(
            client, publicKey, feature, "${stopWatch.elapsedMilliseconds}ms");
      }
      final ChargeResponse cardResponse =
          ChargeResponse.fromJson(jsonDecode(response.body));
      return cardResponse;
    } catch (error) {
      if (feature.isNotEmpty) {
        MetricManager.logMetric(client, publicKey, "{$feature}_ERROR",
            "${stopWatch.elapsedMilliseconds}ms");
      }
      throw (ChApaError(error.toString()));
    }
  }

  /// Verifies payments with Chapa reference
  /// returns an instance of ChargeResponse or throws an error
  static Future<ChargeResponse> verifyPayment(final String chpRef,
      final http.Client client, final String publicKey, final bool isDebugMode,
      [String feature = ""]) async {
    final stopWatch = Stopwatch();

    final url = ChapaURLS.getBaseUrl(isDebugMode) +
        ChapaURLS.VERIFY_TRANSACTION;
    final uri = Uri.parse(url);
    final VerifyChargeRequest verifyRequest = VerifyChargeRequest(chpRef);
    final payload = verifyRequest.toJson();
    try {
      stopWatch.start();
      final http.Response response = await client.post(uri,
          headers: {HttpHeaders.authorizationHeader: publicKey}, body: payload);
      stopWatch.stop();
      final ChargeResponse cardResponse =
          ChargeResponse.fromJson(jsonDecode(response.body));
      if (feature.isNotEmpty) {
        MetricManager.logMetric(
            client, publicKey, feature, "${stopWatch.elapsedMilliseconds}ms");
      }
      return cardResponse;
    } catch (error) {
      if (feature.isNotEmpty) {
        MetricManager.logMetric(client, publicKey, "{$feature}_ERROR",
            "${stopWatch.elapsedMilliseconds}ms");
      }
      throw (ChApaError(error.toString()));
    }
  }
}
